/*
 * $Id: newproc.C,v 1.22 1996/08/15 23:23:38 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995, 1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * Process the initial request to deal with a room:
 * 1) Remove the lock  (assume room is under construction if we can't)
 * 2) Make sure the room is not already marked as being editied, and then
 *    mark the room as open for editing
 * 3) Create a .html file that will tell people that wander into the
 *    room that its under construction.  (This is just temporary.)
 * 4) Create the text entry screen
 */

#include <post-query.H>

#include <htmlfile.H>
#include <htmltext.H>

#include "addvutil.H"
#include "setaddv.H"
#include "undo.H"
#include "openfile.H"
#include "lockfile.H"
#include "browser.H"
#include "error.H"
#include "tokenizer.H"

// Max number of data that can be on the form
#define MAXENTRIES 100

/***************************************************************************/

void hardhat( HtmlFile &f, char *roomword ){
  /*
   * Send to the file text explaining that the room is under construction
   */

  f.title( "Under construction." );
  f.header( 1, "Under construction." );

  f.puts( "Oops... you seem to have wandered into a " );
  f.puts( roomword );
  f.puts( " still being " );
  f.puts( "built.  Try again in a couple of minutes.\n" );

}

/***************************************************************************/

int create_hardhat( int room, char *path, char *roomword ){
  /*
   * Create the file that tells wanderers that this room should be ready
   * soon.
   * Return 1 if success and 0 if error.
   */
  int ret;
  
  FILE *outfile;
  char *filepath;

  // Create the file
  filepath = make_fname( path, room, "html" );
  set_permissions();
  if( outfile = fopen( filepath, "w" ) ){
    // File opened ok
    HtmlFile f( outfile );

    hardhat( f, roomword );
    
    fclose( outfile );
    ret = 1;
  } else {
    // File not opened
    ret = 0;
  }
  
  free( filepath );
  return ret;
}

/***************************************************************************/

void create_textentry( int room, int parent, char *version,
                       int backlink, int listlinks, int dojava,
                       char *roomword,
                       char *filepath, char *urlpath, char *binpath ){
  /*
   * Send a file to the user that contains:
   *   - A title field
   *   - A text entry field
   *   - A field to find out how many options there are
   *   - A way to determine if backlinking is allowed
   */

  char *cgi;     // The executable program
  char *file;    // The executable program with the path of the files
  char *action;  // The URL
  char *undoval;

  // Load filenames for headers and footers
  SetAddv setting( filepath );
  char *headfile = setting.getPathLabel( "file-textentry-head" );
  char *tailfile = setting.getPathLabel( "file-textentry-tail" );

  // Create the form that is inside the page
  HtmlText form;  // The data inside the form

  form.puts( "You may optionally enter a title:" );
  form.text( "title", 60, 0 );  form.para();

  form.puts( "Describe the scenario:\n" );
  form.textarea( "text", 75, 10 );  form.para();

  form.puts( "How many options will the person have? " );
  form.text( "numoptions" );  form.para();

  if( backlink ){
    // We can backlink

    // There is a list of choices that need to be in the form
    HtmlSNode s1( get_backlink_string( BL_NONE, roomword ), 1 );
    HtmlSNode s2( get_backlink_string( BL_NAME, roomword ) );
    HtmlSNode s3( get_backlink_string( BL_NUMBER, roomword ) );
    HtmlSNode s4( get_backlink_string( BL_ENTRY, roomword ) );

    HtmlSList slist;
    slist.append( s1 );
    if( listlinks ){
      // Only allow listing the links in some cases
      slist.append( s2 );
      slist.append( s3 );
    }
    slist.append( s4 );

    form.select( "backlink", &slist );  form.para();

  } else {
    // No backlinking allowed at all
  }
  
  // The "Reject" link
  cgi     = make_fname( binpath, UNDOROOM );
  file    = make_fname( cgi, urlpath );
  Undo undo( room, parent );
  undoval = undo.coded_value();
  action  = make_url( "http", NULL, file, undoval );
  form.link( action, "Cancel" );
  form.puts( " - Do <b>not</b> create this " );
  form.puts( roomword );
  form.puts( ".\n" );
  free( action );
  free( undoval );
  free( file );
  free( cgi );

  // The "Accept" button
  form.submit( "Accept" );
  form.puts( " - Use this description and enter the options." );  form.para();

  // Print the trailer here
  if( tailfile ){
    form.rawfile( tailfile );
    form.para();
  }
  
  // Hidden state values
  form.line();
  form.puts( "Ignore anything below this line.\n" );
  form.hidden( "room", itoa(room) );      form.para();
  form.hidden( "parent", itoa(parent) );  form.para();
  form.hidden( "version", version );      form.para();

  // Convert the form into HTML
  /*
   * The action URL is the name of the processing program, plus
   * the path to the data files.
   *
   * It is necessary to do this separately so we can put this
   * inside an applet tag, or not put it there if specifically
   * requested.
   */
  HtmlText formhtml;
  cgi    = make_fname( binpath, TEXTPROC );
  file   = make_fname( cgi, urlpath );
  action = make_url( "http", NULL, file );
  formhtml.form( action, METHOD_POST, form.val() );
  free( action );
  free( file );
  free( cgi );

  HtmlFile f;     // This goes to the standard output
  f.debug( setting.debug( "newproc-return" ) );

  f.content_type();

  // Create the header and title
  char title[80];
  sprintf( title, "Creating %s %d", roomword, room );
  f.title( title );
  f.header( 1, title );

  // Print a header
  if( headfile ){
    f.para();
    f.rawfile( headfile );
    f.para();
  }

#define DEBUGAPPLET if(setting.debug("newproc-applet"))
#define DEBUGNOAPPLET if(setting.debug("newproc-noapplet"))

  DEBUGAPPLET fprintf( stderr, "enter applet\n" );
  char *elements  = setting.appletelement();
  char *code      = setting.appletelement( "code" );
  char *widthstr  = setting.appletelement( "width" );
  char *heightstr = setting.appletelement( "height" );
  DEBUGAPPLET fprintf( stderr, "dojava = %d\n", dojava );
  if( dojava &&
      null_strlen( elements )  &&
      null_strlen( code )      &&
      null_strlen( widthstr )  &&
      null_strlen( heightstr ) ){
    DEBUGAPPLET fprintf( stderr, "elements=%s\n", elements );
    DEBUGAPPLET fprintf( stderr, "code = %s\n", code );
    DEBUGAPPLET fprintf( stderr, "height = %s\n", heightstr );
    DEBUGAPPLET fprintf( stderr, "width = %s\n", widthstr );
    // Only if any elements are specified then we should
    // start the applet.
    
    int width   = atoi( widthstr );
    int height  = atoi( heightstr );

    // Create the parameter list
    HtmlDList *params = new HtmlDList();
    HtmlDNode *node;

    // Create parameters Addventure wants to set

    DEBUGAPPLET fprintf( stderr, "room=%d\n", room );
    node = new HtmlDNode( "room", null_strdup( itoa( room ) ) );
    params->append( *node );

    DEBUGAPPLET fprintf( stderr, "parent=%d\n", parent );
    node = new HtmlDNode( "parent", null_strdup( itoa( parent ) ) );
    params->append( *node );

    if( backlink ){
      node = new HtmlDNode( "backlink", "true" );
    } else {
      node = new HtmlDNode( "backlink", "false" );
    }
    params->append( *node );

    // No need to send listlinks, since the client can do anything
    // it darn well pleases.

    node = new HtmlDNode( "version", version );
    params->append( *node );

    node = new HtmlDNode( "roomword", roomword );
    params->append( *node );

    node = new HtmlDNode( "urlpath", urlpath );
    params->append( *node );


    // Now load parameters the game admin wants
    char *paramlist = null_strdup( setting.paramname() );
    if( null_strlen( paramlist ) ){
      DEBUGAPPLET fprintf( stderr, "paramlist=%s\n", paramlist );
      char *paramname;
      char *paramvalue;

      int paramlen;
      char *realparamname;

      int ccc = 0;

      // Get the parameters the user wants
      Tokenizer tokens( paramlist );
      paramname = tokens.getnext();
      while( !tokens.finished() ){

        // Rename the parameter so we ensure no collisions
        paramlen = null_strlen( paramname ) + 2;
        realparamname = (char *)malloc( paramlen );
        sprintf( realparamname, "x-%s", paramname );

        // Get the value and add it to the list
        DEBUGAPPLET fprintf( stderr, "param:%s\n", realparamname );
        paramvalue = null_strdup( setting.paramname( paramname ) );
        if( null_strlen( paramvalue ) ){
          // It was in the parameter list, but it was not set itself,
          // so we ignore it.
          DEBUGAPPLET fprintf( stderr, " value:%s\n", paramvalue );
          node = new HtmlDNode( realparamname, paramvalue );
          params->append( *node );
        } else {
          DEBUGAPPLET fprintf( stderr, " no value\n" );
        }

        // Go for the next one
        paramname = tokens.getnext();
      }

      DEBUGAPPLET fprintf( stderr, "ccc=%d\n", ccc );
    }

    // Create the applet stuff with the non-java form inside
    f.applet( code, width, height, params, formhtml.val() );

    // The "Reject" link, without java
    cgi     = make_fname( binpath, UNDOROOM );
    file    = make_fname( cgi, urlpath );
    Undo undoj( room, parent, 1 );
    undoval = undoj.coded_value();
    action  = make_url( "http", NULL, file, undoval );
    f.link( action, "Retry" );
    f.puts( " - Create this " );
    f.puts( roomword );
    f.puts( " without Java.\n" );
    free( action );
    free( undoval );
    free( file );
    free( cgi );

  } else {
    // No legal java info, just do it without the applet tags
    DEBUGNOAPPLET fprintf( stderr, "no java\n" );
    f.raw( formhtml.val() );
    DEBUGNOAPPLET fprintf( stderr, "post raw\n" );
  }
  
}

/***************************************************************************/

main(){

  PostQuery query( MAXENTRIES );

  int room;
  int parent;
  int dojava;      // Should we send java applet info?
  char *version;   // What version was this first established with?
  char *urlpath;   // The path relative to the server root
  char *filepath;  // The absolute path in the file system
  char *binpath;   // Where the cgi scrpts are

  char *browser;   // What browser is in use?

  filepath = getenv( "PATH_TRANSLATED" );
  if( !query.valid() ){
    // Something is wrong
    addventure_error( filepath, NEWPROC, ADDVERR_UNKNOWN );
  } else {
    // Process the data

    // Get the hidden data
    room    = atoi( query.lookup( "room" ) );
    parent  = atoi( query.lookup( "parent" ) );
    if( null_strlen( query.lookup( "nojava" ) ) ){
      // Not defined or is empty, don't do it
      dojava = 0;
    } else {
      // Otherwise, we should
      dojava = 1;
    }
    version = query.lookup( "version" );
    if( !null_strlen( version ) ){
      // This feature wasn't introduced till 2.1
      version = null_strdup( "2.0.x" );
    }
    
    // Get the data passed by environment variables
    urlpath  = getenv( "PATH_INFO" );
    binpath  = get_path( getenv( "SCRIPT_NAME" ) );
    browser = getenv( "HTTP_USER_AGENT" );

    // Store some browser stats
    BrowserFile binfo( filepath );
    binfo.add( room, browser );

    // Get the game settings
    SetAddv settings( filepath );
    char *roomword = settings.roomword();
    int backlink   = settings.backlink();
    int listlinks  = settings.listlinks();

    // Find out if we can actually do this
    OpenFile openfile( filepath, room );
    LockFile lockfile( filepath, room );
    if( !settings.running() ){
      // The game isn't turned on!
      addventure_error( &settings, NEWPROC, ADDVERR_CLOSED );
    } else if( !lockfile.remove() ){
      // We couldn't delete it - room was locked already
      addventure_error( &settings, NEWPROC, ADDVERR_LOCKED,
                        "Lockfile deleted" );
    } else if( openfile.exists() ){
      // Room is already being created
      addventure_error( &settings, NEWPROC, ADDVERR_LOCKED,
                        "Working file exists" );
    } else {
      // We're fine.  Create the file, openfile, and output
      create_hardhat( room, filepath, roomword );
      openfile.create( parent );
      create_textentry( room, parent, version,
                        backlink, listlinks, dojava, roomword, 
                        filepath, urlpath, binpath );
    }
  }
  
}

/***************************************************************************/

/*
 * $Log: newproc.C,v $
 * Revision 1.22  1996/08/15 23:23:38  asf
 * A "Reject" link will display if the applet tags are shown.  This allows
 * a non-java of this page to be created.
 * Some more debugging.
 *
 * Revision 1.21  1996/08/06 02:09:00  asf
 * Removed applet header/footer stuff to take advantage of the new htmlgen
 * routines.
 * Waffling on the parameter to create_textentry()
 * Added chects to see if a value is sent for "nojava" and ensure
 * java shouldn't be displayed.
 * Do applet parameter and param processing.
 *
 * Revision 1.20  1996/07/10 13:11:19  asf
 * Extensive changes to create_textentry() and other locations to
 * begin to support java applets.
 *
 * Revision 1.19  1996/05/03  07:27:01  asf
 * *** empty log message ***
 *
 * Revision 1.18  1996/05/02  21:03:31  asf
 * Standardized to use the error modules instead of the custom
 * error functions.
 *
 * Revision 1.17  1996/03/25  20:11:22  asf
 * Added support for the file-textentry-head and -tail settings to print
 * a header and footer on the page.
 *
 * Revision 1.16  1996/03/24  17:42:49  asf
 * Added a check to see if the game is open and call an error if it isn't.
 *
 * Revision 1.15  1996/02/25  17:46:11  asf
 * "List by name" and "list by number" are now only optionally printed
 *
 * Revision 1.14  1995/01/28  16:56:02  asf
 * Added logging information about browsers used.
 *
 * Expanded the text entry field of the title.
 *
 * Revision 1.13  1995/01/01  16:37:32  asf
 * Added support for the "room word"
 *
 * Revision 1.12  1994/11/06  14:34:54  asf
 * Added parameter to the locked error
 *
 * Revision 1.11  1994/10/01  15:45:34  asf
 * Switched to use of LockFile and OpenFile.
 *
 * Revision 1.10  1994/09/27  17:08:51  asf
 * Added parent parameter to the openfile_create() function
 *
 * Revision 1.9  1994/09/16  15:10:25  asf
 * Do not allow the selection of how to backlink if backlinking is
 * not allowed.
 *
 * Revision 1.8  1994/08/25  01:38:26  asf
 * Standardized accept/reject lines.
 *
 * Revision 1.7  1994/08/25  01:09:23  asf
 * Added openfile support.
 *
 * Revision 1.6  1994/08/20  22:04:59  asf
 * Added reject link
 *
 * Revision 1.5  1994/08/14  14:33:30  asf
 * Added CGI-script name definition
 *
 * Revision 1.4  1994/08/08  20:54:52  asf
 * Added the use of the path to the CGI scripts.
 * Changed some hardcoded strings to call a utility function.
 *
 * Revision 1.3  1994/08/06  09:44:18  asf
 * Added para after the textarea.
 * Textarea should be 75x10, not 10x75
 *
 * Revision 1.2  1994/08/06  09:36:30  asf
 * Ensured that the content type header is sent.
 *
 * Revision 1.1  1994/08/06  08:44:02  asf
 * Initial revision
 *
 */
